@@ -19,6 +19,7 @@
 
 var sid = "";
 var sidToken = "";
+var debug = 0;
 
 // UI components
 var waiting;
@@ -26,8 +27,6 @@ var error;
 
 var model;
 var pModel;
-var tagsModel;
-var navigation;
 
 var continuation = "";
 var actionID = "";
@@ -130,7 +129,7 @@ arrData[ arrData.length - 1 ].push( strMatchedValue );
 return( arrData );
 }
 
-function doWebRequest(method, url, params, callback) {
+function doWebRequest(method, url, params, callback, show) {
     var doc = new XMLHttpRequest();
     //console.log(method + " " + url);
 
@@ -138,25 +137,25 @@ function doWebRequest(method, url, params, callback) {
         if (doc.readyState == XMLHttpRequest.HEADERS_RECEIVED) {
             var status = doc.status;
             if(status!=200) {
-                showError("Google API returned " + status + " " + doc.statusText);
+                showError("API returned " + status + " " + doc.statusText, show);
             }
         } else if (doc.readyState == XMLHttpRequest.DONE) {
             var data;
             var contentType = doc.getResponseHeader("Content-Type");
-            if (params==1) {
+            if (params>=1) {
                 data = doc.responseText;
             }
             else {
                 data = doc.responseXML.documentElement;
             }
-            //var dbg = doc.responseText;
-            //console.log(dbg);
+//            var dbg = doc.responseText;
+//            console.log(dbg);
             callback(data);
         }
     }
 
     doc.open(method, url);
-    if(sid.length>0) {
+    if(sid.length>0 && params<2) {
         // Google Finance ignore SID/LSID
         //console.log("Authorization GoogleLogin auth=" + sid);
         doc.setRequestHeader("Authorization", "GoogleLogin auth=" + sid);
@@ -174,7 +173,7 @@ function doWebRequest(method, url, params, callback) {
     }
 }
 
-function doWebRequestIdx(method, url, params, callback, idx) {
+function doWebRequestIdx(method, url, params, callback, idx, show) {
     var doc = new XMLHttpRequest();
     //console.log(method + " " + url);
 
@@ -182,7 +181,7 @@ function doWebRequestIdx(method, url, params, callback, idx) {
         if (doc.readyState == XMLHttpRequest.HEADERS_RECEIVED) {
             var status = doc.status;
             if(status!=200) {
-                showError("Google API returned " + status + " " + doc.statusText);
+                showError("Google API returned " + status + " " + doc.statusText, show);
             }
         } else if (doc.readyState == XMLHttpRequest.DONE) {
             var data;
@@ -193,6 +192,8 @@ function doWebRequestIdx(method, url, params, callback, idx) {
             else {
                 data = doc.responseXML.documentElement;
             }
+//            var dbg = doc.responseText;
+//            console.log(dbg);
             callback(data, idx);
         }
     }
@@ -216,42 +217,13 @@ function doWebRequestIdx(method, url, params, callback, idx) {
     }
 }
 
-function doYahooWebRequest(method, url, params, callback) {
-    var doc = new XMLHttpRequest();
-    //console.log(method + " " + url);
-
-    doc.onreadystatechange = function() {
-        if (doc.readyState == XMLHttpRequest.HEADERS_RECEIVED) {
-            var status = doc.status;
-            if(status!=200) {
-                showError("Yahoo API returned " + status + " " + doc.statusText);
-            }
-        } else if (doc.readyState == XMLHttpRequest.DONE) {
-            var data;
-            var contentType = doc.getResponseHeader("Content-Type");
-            if (params==1) {
-                data = doc.responseText;
-            }
-            else {
-                data = doc.responseXML.documentElement;
-            }
-            //var dbg = doc.responseText;
-            //console.log(dbg);
-            callback(data);
-        }
-    }
-
-    doc.open(method, url);
-    doc.send();
-}
-
 /** Parse parameter from given URL */
 function parseAuth(data, parameterName) {
     var parameterIndex = data.indexOf(parameterName + "=");
     if(parameterIndex<0) {
         // We didn't find parameter
         console.log("Didn't find Auth");
-        addError("Didn't find Auth");
+        //addError("Didn't find Auth");
         return "";
     }
     var equalIndex = data.indexOf("=", parameterIndex);
@@ -269,14 +241,15 @@ function parseAuth(data, parameterName) {
 function parseToken(data) {
     sid = parseAuth(data, "Auth");
     //console.log("Auth=" + sid);
-    sidToken = parseAuth(data, "SID");
+    //sidToken = parseAuth(data, "SID");
     //console.log("SID=" + sidToken);
 
     if(sid.length>0) {
-        //navigation.state = "menu";
-        waiting.state = "hidden";
+        //waiting.state = "hidden";
+        waiting.state = "portfolio";
     } else {
-        addError("Couldn't parse SID");
+        //addError("Couldn't parse SID");
+        addError("Login failed.\nCheck username/password");
         waiting.state = "hidden";
     }
 }
@@ -285,24 +258,26 @@ function login(email, password) {
     try {
         waiting.state = "login";
         var url = "https://www.google.com/accounts/ClientLogin?Email=" + encodeURIComponent(email) + "&Passwd=" + encodeURIComponent(password) + "&service=finance" + "&source=" + encodeURIComponent("stockona");
-
-        doWebRequest("POST", url, 1, parseToken);
+        doWebRequest("POST", url, 1, parseToken, 0);
     }catch(err) {
-        showError("Error while logging in.");
+        showError("Error while logging in: " + err, 1);
     }
 }
 
-function showError(msg) {
-    // Ignore 500 error
-    waiting.state = "hidden";
+function showError(msg, dbg) {
     console.log(msg);
-    error.reason = msg;
-    error.state = "shown";
+    waiting.state = "hidden";
+
+    if (dbg>0) {
+        error.reason = msg;
+        error.state = "shown";
+    }
 }
+
 function addError(msg) {
     console.log(msg);
     error.reason = msg;
-    error.state = "shownStatic";
+    error.state = "shown";
 }
 
 function loadAllPortfolios() {
@@ -340,19 +315,20 @@ function loadPortfolio() {
     try {
         model.clear();
         //pModel.clear();
+        // Have to set to "portfolio", o.w. stategroup error message
         waiting.state = "shown";
-        doWebRequest("GET", itemsURL, 0, parsePortfolio);
+        doWebRequest("GET", itemsURL, 0, parsePortfolio, 1);
     } catch(err) {
-        showError("Error while loading portfolio: " + err);
+        showError("Error while loading portfolio: " + err, 1);
     }
 }
 
 function loadPosition() {
     try {
         waiting.state = "shown";
-        doWebRequest("GET", itemsURL, 0, parsePosition);
+        doWebRequest("GET", itemsURL, 0, parsePosition, 2);
     } catch(err) {
-        showError("Error while loading position: " + err);
+        showError("Error while loading position: " + err, 1);
     }
 }
 
@@ -367,8 +343,8 @@ function loadQuote(symObj, idx) {
                      "gainPercentFull":symObj.symGainPercent,
                      "rtnYTD":         rtnYTDTrunc,
                      "share":          symObj.symShare,
-                     "shareCostEst":   "0",
-                     "shareValue":     "0",
+                     "shareCost":      symObj.shareCost,
+                     "shareValue":     symObj.shareValue,
 
                      "exchange":       "-",
                      "fullName":       "-",
@@ -403,23 +379,26 @@ function loadQuote(symObj, idx) {
             "share":          symObj.symShare,
             "gainPercentFull":symObj.symGainPercent,
             "rtnYTD":         rtnYTDTrunc, //symObj.symRtnYTD
+            "shareCost":      symObj.shareCost,
+            "shareValue":     symObj.shareValue
+
         });
     }
 
     try {
         waiting.state = "shown";
-        doWebRequestIdx("GET", itemsURL, 1, parseJSON, idx);
+        doWebRequestIdx("GET", itemsURL, 1, parseJSON, idx, 3);
     } catch(err) {
-        showError("Error while loading quote: " + err);
+        showError("Error while loading g_quote: " + err, 1);
     }
 }
 
 function loadYahooQuote() {
     try {
         waiting.state = "shown";
-        doYahooWebRequest("GET", itemsURL, 1, parseCSV);
+        doWebRequest("GET", itemsURL, 2, parseCSV, 3);
     } catch(err) {
-        showError("Error while loading quote: " + err);
+        showError("Error while loading y_quote: " + err, 1);
     }
 }
 
@@ -517,7 +496,8 @@ function parsePosition(data) {
         <gd:feedLink href='http://finance.google.com/finance/feeds/portfolios/2/positions/NYSE:VNQ/transactions'>
                 <feed gd:etag='W/&quot;AkcDRX47eCp7ImA9WhZVEU4.&quot;'>
                         <id>http://finance.google.com/finance/feeds/portfolios/2/positions/NYSE:VNQ/transactions</id>
-                        <updated>2011-05-23T08:07:54.000Z</updated><category scheme='http://schemas.google.com/g/2005#kind' term='http://schemas.google.com/finance/2007#transaction'/>
+                        <updated>2011-05-23T08:07:54.000Z</updated>
+                        <category scheme='http://schemas.google.com/g/2005#kind' term='http://schemas.google.com/finance/2007#transaction'/>
                         <title>Vanguard REIT ETF</title>
                         <link rel='alternate' type='text/html' href='https://finance.google.com/finance/portfolio?action=viewt&amp;pid=0'/>
                         <link rel='http://schemas.google.com/g/2005#feed' type='application/atom+xml' href='http://finance.google.com/finance/feeds/default/portfolios/2/positions'/>
@@ -530,12 +510,12 @@ function parsePosition(data) {
                                 <title>2</title>
                                 <link rel='self' type='application/atom+xml' href='http://finance.google.com/finance/feeds/default/portfolios/2/positions/2'/>
                                 <gf:transactionData shares='0.0' type='Buy'>
-                                <gf:commission>
-                                        <gd:money amount='0.0' currencyCode='USD'/>
-                                </gf:commission>
-                                <gf:price>
-                                        <gd:money amount='0.0' currencyCode='USD'/>
-                                        </gf:price>
+                                    <gf:commission>
+                                            <gd:money amount='0.0' currencyCode='USD'/>
+                                    </gf:commission>
+                                    <gf:price>
+                                            <gd:money amount='0.0' currencyCode='USD'/>
+                                    </gf:price>
                                 </gf:transactionData>
                         </entry>
                 </feed>
@@ -560,30 +540,67 @@ function parsePosition(data) {
         </gf:positionData>
         */
         for (var i=0; i<posLength; i++) {
+            // Transaction data
+            var txLength =data.childNodes[10+i].childNodes[5].firstChild.childNodes[6].firstChild.nodeValue;
 
-            // Broken for now
-            //"symPrice"       : data.childNodes[10+i].childNodes[5].firstChild.childNodes[7].childNodes[7].firstChild.attributes[0].nodeValue,   // per share price
-            //"symCommision"   : data.childNodes[10+i].childNodes[5].firstChild.childNodes[7].childNodes[6].firstChild.attributes[0].nodeValue,   // commission
-            //"symCommisionCurrency"   : data.childNodes[10+i].childNodes[5].firstChild.childNodes[7].childNodes[6].firstChild.attributes[1].nodeValue,   // commission currency
+            // Catch-ya: when there is tx record available, the first gf transactiondata node becomes the date
+            for (var j=0; j<txLength; j=j+1) {
+                // Check attribute number
+                var txTxDataAttLength = data.childNodes[10+i].childNodes[5].firstChild.childNodes[8+j].childNodes[5].attributes.length;
+                var txTxDataOffset = (txTxDataAttLength>=3);
 
-
-            var symObj = {
-            "symId"          : data.childNodes[10+i].childNodes[0].firstChild.nodeValue,
-            "symGainPercent" : data.childNodes[10+i].childNodes[6].attributes[0].nodeValue,
-            "symRtnYTD"      : data.childNodes[10+i].childNodes[6].attributes[8].nodeValue,
-            "symShare"       : data.childNodes[10+i].childNodes[6].attributes[9].nodeValue,
-            "symName"        : data.childNodes[10+i].childNodes[7].attributes[2].nodeValue
-            // Don't parse for now...
-            //"symEtag"        : data.childNodes[10+i].attributes[0].nodeValue,
-            //"symFeedLink"    : data.childNodes[10+i].childNodes[5].attributes[0].nodeValue,   // transaction link
-
-            //"symExg"         : data.childNodes[10+i].childNodes[7].attributes[0].nodeValue,
-            //"symFullName"    : data.childNodes[10+i].childNodes[7].attributes[1].nodeValue,
+                var txObj = {
+                    //"txTrue"        : txTxDataOffset, // 1: Has valid tx data
+                    "txDate"        : data.childNodes[10+i].childNodes[5].firstChild.childNodes[8+j].childNodes[5].attributes[0].nodeValue,
+                    "txShare"       : data.childNodes[10+i].childNodes[5].firstChild.childNodes[8+j].childNodes[5].attributes[0+txTxDataOffset].nodeValue,
+                    "txType"        : data.childNodes[10+i].childNodes[5].firstChild.childNodes[8+j].childNodes[5].attributes[1+txTxDataOffset].nodeValue,
+                    // per share price
+                    "txPrice"       : data.childNodes[10+i].childNodes[5].firstChild.childNodes[8+j].childNodes[5].childNodes[1].firstChild.attributes[0].nodeValue,
+                    "txPriceUnit"   : data.childNodes[10+i].childNodes[5].firstChild.childNodes[8+j].childNodes[5].childNodes[1].firstChild.attributes[1].nodeValue,
+                    // commission
+                    "txCom"         : data.childNodes[10+i].childNodes[5].firstChild.childNodes[8+j].childNodes[5].childNodes[0].firstChild.attributes[0].nodeValue,
+                    "txComUnit"     : data.childNodes[10+i].childNodes[5].firstChild.childNodes[8+j].childNodes[5].childNodes[0].firstChild.attributes[1].nodeValue
             };
 
-            // Inconsistent with documentation, there are no entries.
-            // var symGain        = data.childNodes[10+i].childNodes[6].childNodes[2].attributes[0].nodeValue;
-            // var symMktValue    = data.childNodes[10+i].childNodes[6].childNodes[3].attributes[0].nodeValue;
+//                console.log("####="+symObj.symName);
+//                console.log("txLength "     +   txLength   );
+//                console.log("txAttLength "  +   txTxDataAttLength);
+//                console.log(" txShare "     +  txObj.txShare    );
+//                console.log(" txType "      +  txObj.txType     );
+//                console.log(" txPrice "     +  txObj.txPrice    );
+//                console.log(" txPriceUnit " +  txObj.txPriceUnit);
+//                console.log(" txCom "       +  txObj.txCom      );
+//                console.log(" txComUnit "   +  txObj.txComUnit  );
+            }
+
+            var symObj = {
+                "symId"          : data.childNodes[10+i].childNodes[0].firstChild.nodeValue,
+                "symGainPercent" : data.childNodes[10+i].childNodes[6].attributes[0].nodeValue,
+                "symRtnYTD"      : data.childNodes[10+i].childNodes[6].attributes[8].nodeValue,
+                "symShare"       : data.childNodes[10+i].childNodes[6].attributes[9].nodeValue,
+                "symName"        : data.childNodes[10+i].childNodes[7].attributes[2].nodeValue,
+
+                "shareCost"  : "0",
+                "shareValue" : "0",
+                //"shareGain"  : "0"
+
+                // Don't parse for now...
+                //"symEtag"        : data.childNodes[10+i].attributes[0].nodeValue,
+                //"symFeedLink"    : data.childNodes[10+i].childNodes[5].attributes[0].nodeValue,   // transaction link
+
+                //"symExg"         : data.childNodes[10+i].childNodes[7].attributes[0].nodeValue,
+                //"symFullName"    : data.childNodes[10+i].childNodes[7].attributes[1].nodeValue,
+            };
+
+            // costBasis, gain, mktValue. Only those who have TX records are available
+            if (data.childNodes[10+i].childNodes[6].childNodes.length==4) {
+                symObj.shareCost   = data.childNodes[10+i].childNodes[6].childNodes[0].firstChild.attributes[0].nodeValue;
+                //symObj.shareGain   = data.childNodes[10+i].childNodes[6].childNodes[2].firstChild.attributes[0].nodeValue;
+                symObj.shareValue  = data.childNodes[10+i].childNodes[6].childNodes[3].firstChild.attributes[0].nodeValue;
+                //symObj.symCostUnit = data.childNodes[10+i].childNodes[6].childNodes[0].firstChild.attributes[1].nodeValue;
+            }
+
+            //console.log(symObj.symName + " cost=" + symObj.shareCost + " value=" + symObj.shareValue + " gain=" + symObj.shareGain);
 
             // JSON Parsing, parsed data stored in global jsonObj.
             // Push needs to be done in parseJSON to keep the program synchronous.
@@ -645,38 +662,9 @@ function parseCSV(data) {
 }
 
 function parseJSON(data, idx) {
-    /*
-    "id": "662079"
-    ,"t" : "BRCM"
-    ,"e" : "NASDAQ"
-    ,"l" : "36.52"
-    ,"l_cur" : "36.52"
-    ,"s": "0"
-    ,"ltt":"4:00PM EDT"
-    ,"lt" : "May 27, 4:00PM EDT"
-    ,"c" : "+1.87"
-    ,"cp" : "5.40"
-    ,"ccol" : "chg"
-    ,"eo" : ""
-    ,"delay": ""
-    ,"op" : "35.53"
-    ,"hi" : "36.98"
-    ,"lo" : "35.53"
-    ,"vo" : "23.41M"
-    ,"avvo" : "11.85M"
-    ,"hi52" : "47.39"
-    ,"lo52" : "29.90"
-    ,"mc" : "19.69B"
-    ,"pe" : "18.42"
-    ,"fwpe" : ""
-    ,"beta" : "1.29"
-    ,"eps" : "1.98"
-    ,"name" : "Broadcom Corporation"
-    ,"type" : "Company"
-    */
-
     // Remove comment // at the beginning
-    data = data.replace(/\/\//, "");
+    //data = data.replace(/\/\//, "");
+    data = removeComment(data);
     //console.log(data);
 
     // eval is unsafe, but JSON.parse is prone to fail
@@ -700,26 +688,31 @@ function parseJSON(data, idx) {
 
         // Compute share value and cost for now...
         var gainPtg = pModel.get(idx).gainPercentFull;
-        var share = pModel.get(idx).share;
-        var gainPtgTrunc = sprintf("%.2f", gainPtg*100);
         // truncate precision (or error to be exact)
+        var gainPtgTrunc = sprintf("%.2f", gainPtg*100);
+
+        /*
+        var share = pModel.get(idx).share;
         var shareValue = sprintf("%.4f", share*jsonObj[0].l);
+        console.log(jsonObj[0].t+" "+share+" "+jsonObj[0].l);
+        console.log(jsonObj[0].t+" "+shareValue+" "+share);
 
         // gainPtg can be positive or negative
-        var shareCost;
+        //var shareCost;
         if (gainPtg.match(/^\+/)=="+") {
             // Remove +
             gainPtg = gainPtg.replace(/^\+/, "");
-            shareCost = formatNumber(shareValue/(1+parseFloat(gainPtg)));
+            //shareCost = formatNumber(shareValue/(1+parseFloat(gainPtg)));
         }
         else {
             // Remove -
             gainPtg = gainPtg.replace(/^\-/, "");
-            shareCost = formatNumber(shareValue/(1-parseFloat(gainPtg)));
+            //shareCost = formatNumber(shareValue/(1-parseFloat(gainPtg)));
         }
 
         // Human readable data
         shareValue = formatNumber(share*jsonObj[0].l);
+        */
     }
 
     // Check if entry already exists, if not append, if yes set.
@@ -731,8 +724,6 @@ function parseJSON(data, idx) {
                      "exchange":       (jsonObj[0].e   =="") ? "-" : jsonObj[0].e   ,    //symObj.symExg,
                      "fullName":       (jsonObj[0].name=="") ? "-" : jsonObj[0].name, //symObj.symFullName,
                      "name":           (jsonObj[0].t   =="") ? "-" : jsonObj[0].t   ,    //symObj.symName,
-                     "shareCostEst":   (shareCost      =="") ? "-" : shareCost      ,
-                     "shareValue":     (shareValue     =="") ? "-" : shareValue     ,
                      "gainPercent":    (gainPtgTrunc   =="") ? "-" : gainPtgTrunc   ,
                      "quotePrice":     (jsonObj[0].l   =="") ? "-" : jsonObj[0].l   ,
                      "quoteChgColor":  (quoteChgColor  =="") ? "-" : quoteChgColor  ,
@@ -798,13 +789,6 @@ function getNodeValue(node, name) {
     return nodeValue;
 }
 
-function removeLinks(original) {
-    var txt = original;
-    txt = txt.replace(/<a /g, "<span ");
-    txt = txt.replace(/<\/a>/g, "</span>");
-    return txt;
-}
-
 function formatNumber(data) {
     // > 1M: Display M
     // > 10k: Display K
@@ -822,6 +806,13 @@ function formatNumber(data) {
     }
 }
 
+function removeLinks(original) {
+    var txt = original;
+    txt = txt.replace(/<a /g, "<span ");
+    txt = txt.replace(/<\/a>/g, "</span>");
+    return txt;
+}
+
 // function from http://forums.devshed.com/t39065/s84ded709f924610aa44fff827511aba3.html
 // author appears to be Robert Pollard
 function sprintf()
@@ -985,3 +976,8 @@ function parseDate(stamp)
         return new Date();
     }
 }
+
+function removeComment(data) {
+    data = data.replace(/\/\//, "");
+    return data;
+}
